SystemOrganization addCategory: #'Magma Tools-Buffer Editor'!
SystemOrganization addCategory: #'Magma Tools-Exceptions'!
SystemOrganization addCategory: #'Magma Tools-Health'!
SystemOrganization addCategory: #'Magma Tools-Info'!
SystemOrganization addCategory: #'Magma Tools-Performance'!
SystemOrganization addCategory: #'Magma Tools-Statistics'!

!MaVariableObjectBuffer methodsFor: '*magma tools' stamp: 'cmm 3/19/2007 12:26'!
hasValidReferencesFor: aMaObjectFiler 
	self instVarsDoWithIndex: 
		[ : eachOid : x | 
		(self 
			isValidOid: eachOid
			for: aMaObjectFiler) ifFalse: [ ^ false ] ].
	^ true! !

MagmaUserError subclass: #MagmaToolsUserError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma Tools-Exceptions'!

MaObject subclass: #MagmaBenchmarker
	instanceVariableNames: 'fixture session'
	classVariableNames: 'Port HardwareInfo'
	poolDictionaries: ''
	category: 'Magma Tools-Performance'!

!MagmaBenchmarker class methodsFor: 'private' stamp: 'cmm 3/23/2005 17:14'!
benchName

	^ FileDirectory default fullNameFor: 'MagmaBench'! !

!MagmaBenchmarker class methodsFor: 'hardware info' stamp: 'cmm 3/23/2005 17:24'!
captureHardwareDetails

	| answer |
	[ answer _ 
		FillInTheBlank 
			request: 
				'Please enter a brief description of your hardware separated by commas ', self hardwareAttributes
			initialAnswer: (HardwareInfo ifNil: [ '' ]).
	(answer subStrings: ',') size < (self hardwareAttributes subStrings: ',') size ]
		whileTrue: [ self inform: 'All of those things are important, please enter them all.' ].
	^ answer! !

!MagmaBenchmarker class methodsFor: 'instance creation' stamp: 'cmm 7/5/2008 14:10'!
connectSessionTo: hostString useWriteBarrier: useWriteBarrier 
	| session |
	session _ MagmaSession 
		host: hostString
		port: self port.
	session allowWriteBarrier: useWriteBarrier.
	session connectAs: 'benchTest'.
	^ self session: session! !

!MagmaBenchmarker class methodsFor: 'instance creation' stamp: 'cmm 7/5/2008 14:10'!
forBenchmarkHost: hostString withWriteBarrier: useWriteBarrier 
	"This is a 'one-click' run of the benchmarks that exercises as much of the code as possible, while excluding network latency by using a hostString of 'localhost'."
	^ self 
		connectSessionTo: hostString
		useWriteBarrier: useWriteBarrier! !

!MagmaBenchmarker class methodsFor: 'instance creation' stamp: 'cmm 6/10/2005 16:58'!
forLocalBaseline: useWriteBarrier
	"This is a 'one-click' run of the benchmarks that exercises most of the Magma code without the serialization/materialization cost of Ma client server."

	| session |
	self resetRepository.
	session _ MagmaSession openLocal: self benchName.
	session allowWriteBarrier: useWriteBarrier.
	session connectAs: 'benchTest'.
	^ self session: session! !

!MagmaBenchmarker class methodsFor: 'hardware info' stamp: 'cmm 3/23/2005 17:16'!
hardwareAttributes

	^ 'computer, cpu, speed, memory, disk'! !

!MagmaBenchmarker class methodsFor: 'hardware info' stamp: 'cmm 3/23/2005 17:08'!
hardwareInfo

	^ HardwareInfo _ self captureHardwareDetails! !

!MagmaBenchmarker class methodsFor: 'network baseline' stamp: 'cmm 7/6/2008 20:18'!
port
	"The port the benchmark host will listen on."
	^ Port ifNil: [ Port := 51969 ]! !

!MagmaBenchmarker class methodsFor: 'network baseline' stamp: 'cmm 7/6/2008 20:18'!
port: anInteger
	"Set the port the benchmark host will listen on."
	Port := anInteger! !

!MagmaBenchmarker class methodsFor: 'private' stamp: 'cmm 7/16/2007 22:08'!
resetRepository
	[ MagmaRepositoryController delete: self benchName ] 
		on: MagmaDeleteWarning
		do: [ : warning | warning resume ].
	MagmaRepositoryController 
		create: self benchName
		root: Dictionary new! !

!MagmaBenchmarker class methodsFor: 'network baseline' stamp: 'cmm 7/5/2008 14:10'!
runAgainstBenchmarkHost: hostString withWriteBarrier: useWriteBarrier 
	"This is a 'one-click' run of the benchmarks over a network, the results includes the network latency."
	| benchmarker |
	^ 
	[ (benchmarker _ self 
		forBenchmarkHost: hostString
		withWriteBarrier: useWriteBarrier) runAll ] ensure: [ benchmarker session disconnectAndClose ]! !

!MagmaBenchmarker class methodsFor: 'quick baseline run' stamp: 'cmm 11/13/2006 23:31'!
runLocalBaseline: useWriteBarrier 
	"This is a 'one-click' run of the benchmarks that exercises most of the Magma code without the serialization/materialization cost of Ma client server."
	| benchmarker |
	^ [ (benchmarker _ self forLocalBaseline: useWriteBarrier) runAll ] ensure: [ benchmarker session disconnectAndClose ]! !

!MagmaBenchmarker class methodsFor: 'private' stamp: 'cmm 3/23/2005 23:27'!
runWith: aMagmaSession

	^ (self session: aMagmaSession) runAll! !

!MagmaBenchmarker class methodsFor: 'instance creation' stamp: 'cmm 3/15/2005 22:21'!
session: aMagmaSession

	^ self new
		session: aMagmaSession ;
		yourself! !

!MagmaBenchmarker class methodsFor: 'network baseline' stamp: 'cmm 6/26/2008 10:51'!
startBenchmarkHost
	| console |
	self resetRepository.
	console _ MagmaServerConsole new open: self benchName.
	console processOn: self port.
	console inspect! !

!MagmaBenchmarker methodsFor: 'benchmarks-write' stamp: 'cmm 4/4/2005 13:50'!
a1001BufferCommit
	"How fast can it commit when there is a substantial buffer to write?"

	| toggle |
	toggle _ false.
	session abort; begin.
	session preferences refreshPersistentObjectsEvenWhenChangedOnlyByMe: false. "just in case someone changed the default"
	self focusOn: #oneThousandElementArrayOfObjectsFixture.
	^ [ self oneThousandElementArrayOfObjectsFixture do: [ : eachArray | eachArray at: 1 put: (toggle _ toggle not) ].
	session commitAndBegin ] bench! !

!MagmaBenchmarker methodsFor: 'benchmarks-write' stamp: 'cmm 3/29/2005 14:15'!
aOneMillionBufferCommit
	"How fast can it commit when there is a substantial buffer to write?"

	| toggle |
	toggle _ false.
	session abort; begin.
	session preferences refreshPersistentObjectsEvenWhenChangedOnlyByMe: false. "just in case someone changed the default"
	self focusOn: #oneMillionObjectPointersFixture.
	^ [ self oneMillionObjectPointersFixture do: [ : eachArray | eachArray at: 1 put: (toggle _ toggle not) ].
	session commitAndBegin ] bench! !

!MagmaBenchmarker methodsFor: 'benchmarks-magmacollections' stamp: 'cmm 7/18/2005 11:01'!
addThousandsOfObjectsAtOnce
	"This tests how long it takes to add a fixed number of objects, one at a time, to a MagmaCollection with no indexes."

	self focusOn: #magmaCollectionFixture.
	session abort.  "ensure not in a transaction."
	^ (((Time millisecondsToRun:
		[ session commit:
			[ 1 to: self thousands do:
				[ : n | 
				self magmaCollectionFixture add: Array new ] ] ]) / 1000) asFloat) printString, ' seconds.'! !

!MagmaBenchmarker methodsFor: 'benchmarks-magmacollections' stamp: 'cmm 8/17/2005 22:18'!
addThousandsOfObjectsCommitEvery: batchSize
	"This tests how long it takes to add a fixed number of objects, one at a time, to a MagmaCollection with no indexes."

	"Ensure the collection is already created, so we only measure the add pace."
	session commit: [ self magmaCollectionFixture ].
	self focusOn: #magmaCollectionFixture.
	Smalltalk garbageCollect.
	session
		finalizeOids ;
		abort ;
		begin.
	^ ((Time millisecondsToRun:
		[ 1 to: self thousands do:
			[ : n |
			self magmaCollectionFixture add: Array new.
			n \\ batchSize = 0 ifTrue: [ session commitAndBegin ] ].
		[ session inTransaction ] whileTrue: [ session commit ] ]) / 1000) asFloat printString, ' seconds.'! !

!MagmaBenchmarker methodsFor: 'benchmarks-magmacollections' stamp: 'cmm 8/17/2005 22:24'!
addThousandsOfObjectsOneHundredAtATime
	"This tests how long it takes to add a fixed number of objects to a MagmaCollection with no indexes."

	^ self addThousandsOfObjectsCommitEvery: 100! !

!MagmaBenchmarker methodsFor: 'benchmarks-magmacollections' stamp: 'cmm 8/17/2005 22:19'!
addThousandsOfObjectsTenAtATime
	"This tests how long it takes to add a fixed number of objects to a MagmaCollection with no indexes."

	^ self addThousandsOfObjectsCommitEvery: 10! !

!MagmaBenchmarker methodsFor: 'accessing' stamp: 'cmm 1/11/2007 21:45'!
allCategories
	"These selectors return the array of selectors to run for that category."
	^#( connectionTests readTests writeTests magmaArrayTests magmaCollectionTests )! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 3/15/2005 22:24'!
benchReadOf: oid

	^ [ session realObjectFor: oid ] bench! !

!MagmaBenchmarker methodsFor: 'benchmarks-write' stamp: 'cmm 1/11/2007 21:52'!
connectionTests
	^ #(timeToDisconnect timeToConnect)! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 8/17/2005 22:13'!
focusOn: fixtureSelector

	(fixture keys copyWithout: fixtureSelector) do:
		[ : each |
		session stubOut: (fixture at: each) ].
	Smalltalk garbageCollect.
	session finalizeOids.
	(self perform: fixtureSelector) yourself "ensure proxy-materialized."! !

!MagmaBenchmarker methodsFor: 'benchmarks-magmaarrays' stamp: 'cmm 3/24/2005 00:23'!
getMagmaArraySize

	| array |
	array _ fixture at: #magmaArray.
	^ [ array size ] bench! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 3/24/2005 00:09'!
initializeFixture

	session commit:
		[ session root
			at: 'benchmarkFixture' 
			put: (fixture _ Dictionary new).
		"Force lazy-initialization; it was merely nice to have it with the getter rather than all piled up in one initializer."
		self 
			magmaCollectionFixture ;
			magmaArrayFixture ;
			oneMillionObjectPointersFixture ;
			oneThousandElementArrayFixture ;
			oneThousandElementArrayOfObjectsFixture ;
			singleObjectFixture ;
			thousandLevelDeepFixture ]! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 3/24/2005 00:25'!
magmaArrayFixture

	^ fixture
		at: #magmaArray
		ifAbsentPut: [ MagmaArray new at: 1000 put: -1; yourself ]! !

!MagmaBenchmarker methodsFor: 'benchmarks-magmacollections' stamp: 'cmm 3/24/2005 00:07'!
magmaArrayTests

	^ #( getMagmaArraySize updateThousandsOfMagmaArrayElements )! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 3/13/2005 23:05'!
magmaCollectionFixture

	^ fixture
		at: #magmaCollectionFixture
		ifAbsentPut: [ MagmaCollection new ]! !

!MagmaBenchmarker methodsFor: 'benchmarks-magmacollections' stamp: 'cmm 8/17/2005 22:25'!
magmaCollectionTests

	^ #( addThousandsOfObjectsTenAtATime addThousandsOfObjectsOneHundredAtATime addThousandsOfObjectsAtOnce )! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 6/26/2008 10:49'!
magmaPackageListDo: oneArgBlock 
	^ #(
		'Ma exception handling'
		'Ma base additions'
		'Ma proxy support'
		'Ma traverse object graphs'
		'Ma Statistics'
		'Ma object serialization'
		'Collections-BTree'
		'Ma special collections'
		'Ma client server'
		'WriteBarrier'
		'Magma client'
		'Magma server'
	) do: oneArgBlock! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 3/13/2005 23:06'!
oneMillionObjectPointersFixture

	^ fixture
		at: #oneMillionObjectPointersFixture
		ifAbsentPut:
			[ Array withAll: ((1 to: 1000) collect: [ : x | Array new: 1000 ]) ]! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 3/15/2005 22:22'!
oneMillionObjectPointersRead

	^ self benchReadOf: (session oidFor: self oneMillionObjectPointersFixture)! !

!MagmaBenchmarker methodsFor: 'benchmarks-write' stamp: 'cmm 3/29/2005 14:15'!
oneThousandElementArrayCommit
	"How fast can it commit when there is a substantial buffer to write?"

	| toggle |
	toggle _ false.
	session abort; begin.
	session preferences refreshPersistentObjectsEvenWhenChangedOnlyByMe: false. "just in case someone changed the default"
	self focusOn: #oneThousandElementArrayFixture.
	^ [ self oneThousandElementArrayFixture at: 1 put: (toggle _ toggle not).
	session commitAndBegin ] bench! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 3/13/2005 23:06'!
oneThousandElementArrayFixture

	^ fixture
		at: #oneThousandElementArrayFixture
		ifAbsentPut: [ Array new: 1000 ]! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 3/13/2005 23:06'!
oneThousandElementArrayOfObjectsFixture

	^ fixture
		at: #oneThousandElementArrayOfObjectsFixture
		ifAbsentPut:
			[ Array withAll: ((1 to: 1000) collect: [ : x | Array new: 1 ]) ]! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 3/15/2005 22:24'!
oneThousandElementArrayOfObjectsRead

	^ self benchReadOf: (session oidFor: self oneThousandElementArrayOfObjectsFixture)! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 3/15/2005 22:23'!
oneThousandElementArrayRead

	^ self benchReadOf: (session oidFor: self oneThousandElementArrayFixture)! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 3/15/2005 22:22'!
oneThousandLevelsDeepRead

	^ self benchReadOf: (session oidFor: self thousandLevelDeepFixture)! !

!MagmaBenchmarker methodsFor: 'benchmarks-write' stamp: 'cmm 3/29/2005 14:15'!
peakCommitRate
	"A minimal commit-package committed over and over to see what the fastest Magma's architecture can oscillate.  This is done using auto-begin to shave the last bit of efficiency."

	| toggle |
	toggle _ false.
	session abort; begin.
	session preferences refreshPersistentObjectsEvenWhenChangedOnlyByMe: false. "just in case someone changed the default"
	self focusOn: #singleObjectFixture.
	^ [ self singleObjectFixture at: 1 put: (toggle _ toggle not).
	session commitAndBegin ] bench! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 3/24/2005 00:05'!
peakRefreshRate

	^ [ session abort ] bench! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 3/24/2005 00:05'!
readTests

	^ #(peakRefreshRate singleObjectRead oneThousandElementArrayRead oneThousandElementArrayOfObjectsRead oneMillionObjectPointersRead oneThousandLevelsDeepRead)! !

!MagmaBenchmarker methodsFor: 'run' stamp: 'cmm 6/27/2008 00:24'!
run: selector resultsTo: aStream 
	aStream
		cr ;
		tab ;
		nextPutAll: selector ;
		nextPutAll: ' : ' ;
		nextPutAll: (self perform: selector).
	"Set each MaHistoricalStat's name to refer to this test, so we can see which test this stat is for."
	session statistics historysDo: 
		[ : eachStatHistory | 
		eachStatHistory isActive ifTrue: [ eachStatHistory active name: selector ] ].
	session statistics transition! !

!MagmaBenchmarker methodsFor: 'run' stamp: 'cmm 3/23/2005 22:38'!
runAll

	^ self runAll: self allCategories! !

!MagmaBenchmarker methodsFor: 'run' stamp: 'cmm 6/26/2008 22:20'!
runAll: categoriesArray 
	| answer |
	answer := WriteStream on: String new.
	self
		writeContextInformationTo: answer ;
		initializeFixture.
	session readStrategy: MaReadStrategy deep.
	answer
		cr ;
		cr ;
		nextPutAll: 'Benchmarks:'.
	Smalltalk garbageCollect.
	categoriesArray do: 
		[ : each | 
		self 
			runCategory: each
			resultsTo: answer ].
	session disconnect.
	answer
		cr ;
		cr ;
		nextPutAll: '*** Client Session Statistics Report ***' ;
		cr.
	session statistics printReportTo: answer.
	answer
		cr ;
		cr ;
		nextPutAll: '*** Server Statistics Report ***' ;
		cr.
	session serverStatistics printReportTo: answer.
	^ answer contents! !

!MagmaBenchmarker methodsFor: 'run' stamp: 'cmm 3/13/2005 23:03'!
runCategory: selector resultsTo: aStream

	aStream
		cr; nextPutAll: '---';
		cr ;
		cr; nextPutAll: selector.
	(self perform: selector) do:
		[ : each |
		self
			run: each
			resultsTo: aStream ]! !

!MagmaBenchmarker methodsFor: 'accessing' stamp: 'cmm 6/10/2005 17:01'!
session
	^ session! !

!MagmaBenchmarker methodsFor: 'initialize-release' stamp: 'cmm 3/15/2005 22:21'!
session: aMagmaSession

	session _ aMagmaSession! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 3/13/2005 23:06'!
singleObjectFixture

	^ fixture
		at: #singleObjectFixture
		ifAbsentPut: [ Array new: 1 ]! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 3/15/2005 22:22'!
singleObjectRead

	^ self benchReadOf: (session oidFor: self singleObjectFixture)! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 3/13/2005 23:06'!
thousandLevelDeepFixture

	^ fixture
		at: #thousandLevelDeepFixture
		ifAbsentPut:
			[ | rootOfDeepObject |
			rootOfDeepObject _ Array new: 1.
			(1 to: 1000)
				inject: rootOfDeepObject
				into:
					[ : prior : x |
					prior at: 1 put: (Array new: 1) ].
			rootOfDeepObject ]! !

!MagmaBenchmarker methodsFor: 'accessing' stamp: 'cmm 3/23/2005 22:37'!
thousands
	"The size of the MagmaCollection tests."

	^ 1000! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 6/26/2008 22:59'!
timeToConnect
	"We do a manual #transition after every test, so allow up to 250 in the queue, none of which will be based on having transitioned due to time."
	session statistics snapshotInterval: (10 
			to: 2500
			by: 10).
	^ [ session connectAs: 'benchmarker' ] durationToRun printString! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 1/11/2007 21:52'!
timeToDisconnect
	^ [ session disconnect ] durationToRun printString! !

!MagmaBenchmarker methodsFor: 'benchmarks-magmaarrays' stamp: 'cmm 11/14/2006 21:14'!
updateThousandsOfMagmaArrayElements
	| array |
	array _ fixture at: #magmaArray.
	session begin.
	^ 
	[ 
	[ 1 
		to: self thousands
		do: 
			[ : x | 
			array 
				at: x
				put: x ].
	session commitAndBegin ] bench ] ensure: [ session abort ]! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 3/23/2005 23:47'!
writeBenchmarkerInformationTo: aStream

	aStream
		cr; cr;
		nextPutAll: 'Benchmarker Details:'.
	#(thousands)
		do:
			[ : each | 
			self
				writeEntry: each
				value: (self perform: each)
				to: aStream ]! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 4/8/2005 13:11'!
writeClientSessionInformationTo: aStream

	aStream
		cr; cr;
		nextPutAll: 'MagmaSession Details:'.
	#(isLocal)
		do:
			[ : each | 
			self
				writeEntry: each
				value: (session perform: each)
				to: aStream ].
	#(allowWriteBarrier refreshPersistentObjectsEvenWhenChangedOnlyByMe)
		do:
			[ : each | 
			self
				writeEntry: each
				value: (session preferences perform: each)
				to: aStream ]! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 9/6/2007 21:28'!
writeContextInformationTo: aStream 
	aStream nextPutAll: 'The date is '.
	DateAndTime now printOn: aStream.
	self
		writeHardwareInformationTo: aStream ;
		writeOsInformationTo: aStream ;
		writeImageInformationTo: aStream ;
		writePackageInformationTo: aStream ;
		writeServerInformationTo: aStream ;
		writeClientSessionInformationTo: aStream ;
		writeBenchmarkerInformationTo: aStream! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 6/16/2005 23:43'!
writeEntry: selectorSymbol value: anObject to: aStream

	aStream
		cr; tab;
		nextPutAll: selectorSymbol withBlanksTrimmed ;
		nextPutAll: ' : ' ;
		maPrint: anObject! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 3/13/2005 18:14'!
writeHardwareInformationTo: aStream

	aStream
		cr ; cr ;
		nextPutAll: 'Hardware Details:'.
	(self class hardwareAttributes subStrings: ',') with: (self class hardwareInfo subStrings: ',') do:
		[ : eachAttribute : eachValue |
		self
			writeEntry: eachAttribute
			value: eachValue
			to: aStream ]! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 3/14/2005 22:06'!
writeImageInformationTo: aStream

	aStream
		cr; cr;
		nextPutAll: 'Image Details:'.
	self
		writeEntry: 'version'
		value: SystemVersion current version
		to: aStream.
	self
		writeEntry: 'lastUpdate'
		value: SystemVersion current highestUpdate printString
		to: aStream! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 3/14/2005 22:04'!
writeOsInformationTo: aStream

	aStream
		cr; cr;
		nextPutAll: 'OS Details:'.
	#(osVersion platformName platformSubtype vmVersion imageName)
		do:
			[ : each | 
			self
				writeEntry: each
				value: (SmalltalkImage current perform: each)
				to: aStream ]! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 6/26/2008 23:31'!
writePackageInformationTo: aStream 
	| workingCopy versionInformation |
	aStream
		cr ;
		cr ;
		nextPutAll: 'Code Package Details (from Monticello):'.
	self magmaPackageListDo: 
		[ : each | 
		workingCopy := (MCPackage named: each) workingCopy.
		versionInformation := workingCopy ancestors first.
		aStream
			cr ;
			cr ;
			nextPutAll: versionInformation summaryHeader.
		workingCopy needsSaving ifTrue: 
			[ aStream
				cr ;
				tab ;
				nextPutAll: '** modified **' ] ]! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 8/17/2005 23:01'!
writeServerInformationTo: aStream

	session isLocal ifFalse: [ ^ self "may want to figure out how to do this some day." ].
	aStream
		cr; cr;
		nextPutAll: 'Magma server details:'.
	#(flushAggressively)
		do:
			[ : each | 
			self
				writeEntry: each
				value: (session repositoryController perform: each)
				to: aStream ]! !

!MagmaBenchmarker methodsFor: 'benchmarks-write' stamp: 'cmm 3/13/2005 23:24'!
writeTests

	^ #(peakCommitRate oneThousandElementArrayCommit a1001BufferCommit aOneMillionBufferCommit)! !

MaObject subclass: #MagmaBufferController
	instanceVariableNames: 'repositoryController cache classMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma Tools-Buffer Editor'!

!MagmaBufferController commentStamp: 'cmm 3/22/2005 22:28' prior: 0!
I provide a means for browsing and editing buffers stored in a Magma repository.  Hopefully, I'll never be needed.  But if you need to perform surgery on a repository, I am a great helper.

I help by presenting the domain-model as a graph of MagmaBufferFacades.  Each facade represents exactly one MaObjectBuffer, which is the primary unit-of-storage for a single object stored in Magma.  The facade provides as much information about the real object-model as possible.  Each facade provides #bufferVarAt: to mimic #instVarAt: of regular Objects.  In this manner, you may see the "shape" of the domain-model.

Should you be unable to open a repository (a primary reason for needing me in the first place), you may build up my classMap as you uncover class-ids using clues from the buffers (for example, since all String buffers are automatically shown, you may recognize a reference to a particular String and identify the class-id of its referencer that way).

My 'cache' ensures facades are reused as much as possible.

To use, you supply an open MagmaRepositoryController.  All of my read-operations are thread-safe; you can use this even when the repository is serving other clients.  My write-operations, however, are NOT transaction-safe, i.e., since we're not going through a session, there is no commit-conflict checking.
!

!MagmaBufferController class methodsFor: 'create' stamp: 'cmm 3/22/2005 22:11'!
repositoryController: aMagmaRepositoryController
	^ self new
		repositoryController: aMagmaRepositoryController ;
		yourself! !

!MagmaBufferController methodsFor: 'root buffers' stamp: 'cmm 3/22/2005 23:19'!
anchorBuffer

	^ self bufferAt: self filer anchorOid! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 6/1/2005 22:42'!
bufferAt: anInteger

	^ cache
		at: { repositoryController. anInteger }
		ifAbsentPut:
			[ MagmaBufferFacade
				buffer: (self readAt: anInteger)
				controller: self ]! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 3/23/2005 13:29'!
classDefinitionFor: classIdInteger version: versionInteger

	^ classMap
		at: { classIdInteger. versionInteger }
		ifAbsent:
			[ [ self classIdManager
				definitionForClassId: classIdInteger
				version: versionInteger ]
			
				on: Error
				do: 
					[ : err | 
					(MaClassDefinition className: '(id=', classIdInteger printString, ', v=', versionInteger printString, ') unknown')
						id: classIdInteger ;
						version: versionInteger ;
						yourself ] ]! !

!MagmaBufferController methodsFor: 'root buffers' stamp: 'cmm 3/22/2005 23:19'!
classDefinitionsBuffer

	^ self bufferAt: self filer classDefinitionsOid! !

!MagmaBufferController methodsFor: 'private' stamp: 'cmm 3/22/2005 23:25'!
classIdManager

	^ self serializer classIdManager! !

!MagmaBufferController methodsFor: 'private' stamp: 'cmm 3/22/2005 23:08'!
filer
	^ self repository filer! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 3/22/2005 23:36'!
idOfClassNamed: aString is: anInteger

	classMap
		at: anInteger
		put: aString! !

!MagmaBufferController methodsFor: 'initializing' stamp: 'cmm 6/1/2005 22:42'!
initialize
	super initialize.
	cache _ Dictionary new! !

!MagmaBufferController methodsFor: 'initializing' stamp: 'cmm 6/1/2005 22:39'!
initializeClassMap
	classMap _ Dictionary new.
	[ self classIdManager classDefinitionsById do:
		[ : each |
		classMap at: each anyOne id put: each ] ]

		on: Error
		do:
			[ : err | 
			classMap _ Dictionary new.
			MagmaToolsWarning signal: 'Could not even load the class-definitions.  You will have to try to guess the classes yourself.  You may use #idOfClassNamed:is: to build it up.' ]! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 6/1/2005 22:55'!
objectClassNameFor: classIdInteger

	^ classMap
		maAt: classIdInteger
		ifPresent: [ : versions | versions anyOne name ]
		ifAbsent:
			[ (self classIdManager
				classForId: classIdInteger
				ifAbsent:
					[ classIdInteger, ' - unknown' ]) asString ]! !

!MagmaBufferController methodsFor: 'private' stamp: 'cmm 3/22/2005 23:09'!
readAt: oidInteger

	^ repositoryController requestCritical: [ self filer bufferFor: oidInteger ]! !

!MagmaBufferController methodsFor: 'private' stamp: 'cmm 3/22/2005 23:08'!
repository
	^ repositoryController repository! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 6/1/2005 22:44'!
repositoryController
	^ repositoryController! !

!MagmaBufferController methodsFor: 'initializing' stamp: 'cmm 6/1/2005 22:37'!
repositoryController: aMagmaRepositoryController
	repositoryController _ aMagmaRepositoryController.
	self initializeClassMap! !

!MagmaBufferController methodsFor: 'root buffers' stamp: 'cmm 3/22/2005 23:20'!
repositoryDefinitionBuffer

	^ self bufferAt: self filer definitionOid! !

!MagmaBufferController methodsFor: 'private' stamp: 'cmm 3/22/2005 23:25'!
serializer

	^ repositoryController serverSerializer! !

MaObject subclass: #MagmaBufferFacade
	instanceVariableNames: 'buffer controller annotation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma Tools-Buffer Editor'!

!MagmaBufferFacade class methodsFor: 'create' stamp: 'cmm 3/22/2005 22:47'!
buffer: aMaObjectBuffer controller: aMagmaRepositoryController

	^ self new
		buffer: aMaObjectBuffer ;
		controller: aMagmaRepositoryController ;
		yourself! !

!MagmaBufferFacade methodsFor: 'initializing' stamp: 'cmm 3/22/2005 22:47'!
buffer: aMaObjectBuffer
	buffer _ aMaObjectBuffer! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/22/2005 23:44'!
bufferAt: anInteger
	"Answer the buffer I reference at slot anInteger."

	buffer isPointers ifFalse: [ MagmaToolsUserError signal: 'no references to other buffers' ].
	^ MaOidCalculator
		objectWithOid: anInteger
		ifNone: [ controller bufferAt: anInteger ]! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/22/2005 23:12'!
classId
	^ buffer classId! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/22/2005 23:13'!
classVersion
	^ buffer classVersion! !

!MagmaBufferFacade methodsFor: 'initializing' stamp: 'cmm 3/23/2005 13:10'!
controller: aMagmaBufferController
	controller _ aMagmaBufferController! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/22/2005 23:24'!
objectClassName
	^ controller objectClassNameFor: self classId! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/22/2005 23:12'!
oid
	^ buffer oid! !

!MagmaBufferFacade methodsFor: 'printing' stamp: 'cmm 3/22/2005 23:40'!
printOn: aStream

	super printOn: aStream.
	buffer isBytes
		ifTrue:
			[ aStream nextPutAll: buffer asString ]
		ifFalse:
			[ self
				maPrintAttributes: #(oid objectClassName)
				on: aStream ]! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/22/2005 23:53'!
referencedBuffers

	^ (1 to: buffer objectInstSize) collect:
		[ : index |
		self bufferAt: (buffer maInstVarAt: index) ]! !

MaObject subclass: #MagmaDoctor
	instanceVariableNames: 'filer invalidClassDefinitionsBuffer invalidDefinitionBuffer invalidAnchorBuffer classIdOfDictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma Tools-Health'!

!MagmaDoctor commentStamp: 'cmm 3/25/2007 19:05' prior: 0!
I present a report on a MagmaRepository.  I open the repository specified, report all of its primitive-attributes, then the contents and state of the crucial buffers (classDefinitions, anchor, definition).  

If any of them do not look correct, I perform a scan of the object file, looking for them.
!

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 14:19'!
dataAttributes
	^ { }! !

!MagmaDoctor methodsFor: 'private' stamp: 'cmm 5/11/2007 14:40'!
filer
	^ filer! !

!MagmaDoctor methodsFor: 'initialize-release' stamp: 'cmm 3/25/2007 19:18'!
filer: aMaObjectFiler
	filer _ aMaObjectFiler! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 14:16'!
headerAttributes
	^ {  (TextEmphasis bold). (TextEmphasis underlined)  }! !

!MagmaDoctor methodsFor: 'initialize-release' stamp: 'cmm 3/29/2007 15:21'!
initialize
	super initialize.
	invalidAnchorBuffer _ invalidClassDefinitionsBuffer _ invalidDefinitionBuffer _ false! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 14:29'!
invalid
	^ '**INVALID**'! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 14:18'!
labelAttributes
	^ {  (TextEmphasis bold)  }! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:18'!
looksLikeAnchorBuffer: aMaObjectBuffer 
	| stream |
	stream _ WriteStream on: String new.
	self 
		printProblemsWithAnchorBuffer: aMaObjectBuffer
		on: stream.
	^ stream isEmpty! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/26/2007 23:01'!
looksLikeClassDefinitionBuffer: aMaObjectBuffer
	| stream |
	stream _ WriteStream on: String new.
	self printProblemsWithClassDefinitionsBuffer: aMaObjectBuffer on: stream.
	^ stream isEmpty! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:13'!
looksLikeDefinitionBuffer: aMaObjectBuffer 
	| stream |
	stream _ WriteStream on: String new.
	self 
		printProblemsWithDefinitionBuffer: aMaObjectBuffer
		on: stream.
	^ stream isEmpty! !

!MagmaDoctor methodsFor: 'accessing' stamp: 'cmm 3/25/2007 19:19'!
openHealthReport
	| ts |
	ts _ TextStream on: Text new.
	self writeHealthInformationTo: ts.
	(Workspace new
		contents: ts contents ;
		yourself) openLabel: self pathName! !

!MagmaDoctor methodsFor: 'accessing' stamp: 'cmm 5/11/2007 13:10'!
pathName
	^ filer directory pathName! !

!MagmaDoctor methodsFor: 'private' stamp: 'cmm 3/29/2007 15:14'!
potentialAnchorBuffers
	| potentialAnswers |
	potentialAnswers _ OrderedCollection new.
	self filer objectBuffersDo: 
		[ : each | 
		(self looksLikeAnchorBuffer: each) ifTrue: [ potentialAnswers add: each ] ].
	^ potentialAnswers! !

!MagmaDoctor methodsFor: 'private' stamp: 'cmm 3/29/2007 15:38'!
potentialClassDefinitionBuffers
	| potentialAnswers |
	potentialAnswers _ OrderedCollection new.
	self filer objectBuffersDo: 
		[ : each | 
		(self looksLikeClassDefinitionBuffer: each) ifTrue: [ potentialAnswers add: each ] ].
	(potentialAnswers collect: [ : e | e classId ]) asSet in: [ : classIds | classIds size = 1 ifTrue: [ classIdOfDictionary _ classIds anyOne ] ].
	^ potentialAnswers! !

!MagmaDoctor methodsFor: 'private' stamp: 'cmm 3/29/2007 15:14'!
potentialDefinitionBuffers
	| potentialAnswers |
	potentialAnswers _ OrderedCollection new.
	self filer objectBuffersDo: 
		[ : each | 
		(self looksLikeDefinitionBuffer: each) ifTrue: [ potentialAnswers add: each ] ].
	^ potentialAnswers! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:46'!
printAnchorBufferOn: aWriteStream 
	| buffer |
	self 
		printHeading: 'The Anchor Buffer'
		on: aWriteStream.
	buffer _ self filer bufferFor: self filer anchorOid.
	self 
		printFieldNamed: 'oid'
		value: buffer oid
		on: aWriteStream.
	self 
		printFieldNamed: 'bufferType'
		value: buffer bufferType
		on: aWriteStream.
	self 
		printFieldNamed: #objectInstSize
		value: buffer objectInstSize
		on: aWriteStream.
	self 
		printFieldNamed: #classId
		value: buffer classId
		on: aWriteStream.
	self 
		printProblemsWithAnchorBuffer: buffer
		on: aWriteStream.
"	invalidAnchorBuffer ifTrue: [ self printPotentialAnchorBuffersOn: aWriteStream ]"! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:12'!
printClassDefinitionsBufferOn: aWriteStream 
	| buffer |
	self 
		printHeading: 'The Class-Definitions Buffer'
		on: aWriteStream.
	buffer _ self filer bufferFor: self filer classDefinitionsOid.
	self 
		printFieldNamed: 'oid'
		value: buffer oid
		on: aWriteStream.
	self 
		printFieldNamed: 'bufferType'
		value: buffer bufferType
		on: aWriteStream.
	self 
		printFieldNamed: #objectInstSize
		value: buffer objectInstSize
		on: aWriteStream.
	self 
		printFieldNamed: #classId
		value: buffer classId
		on: aWriteStream.
	self 
		printProblemsWithClassDefinitionsBuffer: buffer
		on: aWriteStream.
	invalidClassDefinitionsBuffer ifTrue: [ self printPotentialClassDefinitionsBuffersOn: aWriteStream ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 19:04'!
printCrucialBuffersOn: aWriteStream 
	self
		printClassDefinitionsBufferOn: aWriteStream ;
		printDefinitionBufferOn: aWriteStream ;
		printAnchorBufferOn: aWriteStream! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:49'!
printDefinitionBufferOn: aWriteStream 
	| buffer |
	self 
		printHeading: 'The Repository-Definition Buffer'
		on: aWriteStream.
	buffer _ self filer bufferFor: self filer definitionOid.
	self 
		printFieldNamed: 'oid'
		value: buffer oid
		on: aWriteStream.
	self 
		printFieldNamed: 'bufferType'
		value: buffer bufferType
		on: aWriteStream.
	self 
		printFieldNamed: #objectInstSize
		value: buffer objectInstSize
		on: aWriteStream.
	self 
		printFieldNamed: #classId
		value: buffer classId
		on: aWriteStream.
	self 
		printProblemsWithDefinitionBuffer: buffer
		on: aWriteStream.
	"invalidDefinitionBuffer ifTrue: [ self printPotentialDefinitionBuffersOn: aWriteStream ]"! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/26/2007 22:56'!
printFieldNamed: aString value: anObject on: aWriteStream 
	self 
		printLabel: aString
		value: anObject
		withAttributes: self dataAttributes
		on: aWriteStream! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 19:24'!
printHeading: aString on: aWriteStream 
	aWriteStream
		cr ;
		cr ;
		withAttributes: self headerAttributes
			do: [ aWriteStream maPrint: aString ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/26/2007 22:57'!
printInvalidOn: aWriteStream withMessage: aString 
	self 
		printLabel: self invalid
		value: aString
		withAttributes: self problemAttributes
		on: aWriteStream! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/26/2007 22:56'!
printLabel: aString value: anObject withAttributes: attributes on: aWriteStream 
	aWriteStream 
		withAttributes: attributes
		do: 
			[ aWriteStream
				cr ;
				tab ;
				maPrint: aString ;
				maPrint: ':  ' ].
	aWriteStream 
		withAttributes: self dataAttributes
		do: [ aWriteStream maPrint: anObject ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:28'!
printPotentialAnchorBuffersOn: aWriteStream 
	self 
		printHeading: 'Candidates for the real AnchorBuffer'
		on: aWriteStream.
	self potentialAnchorBuffers do: 
		[ : each | 
		aWriteStream
			cr ;
			tab ;
			maPrint: each ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:36'!
printPotentialClassDefinitionsBuffersOn: aWriteStream 
	self 
		printHeading: 'Candidates for the real ClassDefinitions buffer'
		on: aWriteStream.
	self potentialClassDefinitionBuffers do: 
		[ : each | 
		aWriteStream
			cr ;
			tab ;
			maPrint: each ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:36'!
printPotentialDefinitionBuffersOn: aWriteStream 
	self 
		printHeading: 'Candidates for the real Definition buffer'
		on: aWriteStream.
	self potentialDefinitionBuffers do: 
		[ : each | 
		aWriteStream
			cr ;
			tab ;
			maPrint: each ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 14:51'!
printPrimitiveAttributesOn: aWriteStream 
	self 
		printHeading: 'Primitive Repository Values'
		on: aWriteStream.
	self
		printSignatureOn: aWriteStream ;
		printVersionOn: aWriteStream.
	#(#classDefinitionsOid #definitionOid #anchorOid ) do: 
		[ : each | 
		self 
			printSpecialOid: each
			on: aWriteStream ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:20'!
printProblemsWithAnchorBuffer: aMaObjectBuffer on: aWriteStream 
	aMaObjectBuffer bufferType = MaFixedObjectBuffer basicNew bufferType ifFalse: 
		[ invalidAnchorBuffer _ true.
		self 
			printInvalidOn: aWriteStream
			withMessage: 'The buffer should be a MaFixedObjectBuffer, it isn''t.' ].
	aMaObjectBuffer objectInstSize = MaRootAnchor instSize ifFalse: 
		[ invalidAnchorBuffer _ true.
		self 
			printInvalidOn: aWriteStream
			withMessage: 'This buffer should represent a MaRootAnchor, but its instSize isn''t right for that' ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:08'!
printProblemsWithClassDefinitionsBuffer: aMaObjectBuffer on: aWriteStream 
	| indexOfNonIntegerClassId indexOfInvalidUserOidClassDef |
	aMaObjectBuffer bufferType = MaVariableObjectBuffer basicNew bufferType ifFalse: 
		[ invalidClassDefinitionsBuffer _ true.
		self 
			printInvalidOn: aWriteStream
			withMessage: 'The buffer should be a MaVariableObjectBuffer, it isn''t.'.
		^ self ].
	(aMaObjectBuffer objectInstSize even and: [ aMaObjectBuffer objectInstSize > 0 ]) ifFalse: 
		[ invalidClassDefinitionsBuffer _ true.
		self 
			printInvalidOn: aWriteStream
			withMessage: 'This buffer should represent a Dictionary, so it should always have an even objectInstSize (key->value pairs).'.
		^ self ].
	indexOfNonIntegerClassId _ 0.
	indexOfInvalidUserOidClassDef _ 0.
	aMaObjectBuffer instVarsDoWithIndex: 
		[ : eachOid : index | 
		index odd 
			ifTrue: 
				[ (MaOidCalculator isOidForOptimizedInteger: eachOid) ifFalse: [ indexOfNonIntegerClassId _ index ] ]
			ifFalse: 
				[ (MaOidCalculator isOidForUserObject: eachOid) ifFalse: [ indexOfInvalidUserOidClassDef _ index ] ] ].
	indexOfNonIntegerClassId = 0 ifFalse: 
		[ invalidClassDefinitionsBuffer _ true.
		self 
			printInvalidOn: aWriteStream
			withMessage: 'The class-definitions buffer should be a Dictionary with SmallIntegers as its keys.  This is not the case at instVar' , indexOfNonIntegerClassId printString ].
	indexOfInvalidUserOidClassDef = 0 ifFalse: 
		[ invalidClassDefinitionsBuffer _ true.
		self 
			printInvalidOn: aWriteStream
			withMessage: 'The class-definitions buffer should be a Dictionary user objects (MaClassDefinitions) as its values.  This is not the case at instVar' , indexOfInvalidUserOidClassDef printString ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:12'!
printProblemsWithDefinitionBuffer: aMaObjectBuffer on: aWriteStream 
	aMaObjectBuffer bufferType = MaFixedObjectBuffer basicNew bufferType ifFalse: 
		[ invalidDefinitionBuffer _ true.
		self 
			printInvalidOn: aWriteStream
			withMessage: 'The buffer should be a MaFixedObjectBuffer, it isn''t.' ].
	aMaObjectBuffer objectInstSize = MagmaRepositoryDefinition instSize ifFalse: 
		[ invalidDefinitionBuffer _ true.
		self 
			printInvalidOn: aWriteStream
			withMessage: 'This buffer should represent a MagmaRepositoryDefinition, but its instSize isn''t right for that' ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 13:31'!
printRepositoryPathOn: aWriteStream 
	aWriteStream
		cr ;
		maPrint: 'pathName:  ' ;
		maPrint: self pathName! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 19:21'!
printSignatureOn: aWriteStream 
	self 
		printFieldNamed: 'signature'
		value: self filer signature
		on: aWriteStream.
	[ self filer validateSignature ] 
		on: MagmaUserError
		do: 
			[ : err | 
			self 
				printInvalidOn: aWriteStream
				withMessage: err messageText ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 19:31'!
printSpecialOid: selectorSymbol on: aWriteStream 
	self 
		printFieldNamed: selectorSymbol
		value: (self filer perform: selectorSymbol)
		on: aWriteStream.
	[ self filer validateSpecialOid: selectorSymbol ] 
		on: MagmaCorruptionError
		do: 
			[ : err | 
			self 
				printInvalidOn: aWriteStream
				withMessage: err messageText ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 14:53'!
printTitleOn: aWriteStream 
	aWriteStream 
		withAttributes: { 
				(TextAlignment centered).
				(TextEmphasis underlined)
			 }
		do: [ aWriteStream maPrint: self pathName ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 19:21'!
printVersionOn: aWriteStream 
	self 
		printFieldNamed: 'version'
		value: self filer version
		on: aWriteStream.
	[ self filer validateVersion ] 
		on: MagmaUserError
		do: 
			[ : err | 
			self 
				printInvalidOn: aWriteStream
				withMessage: err messageText ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 14:20'!
problemAttributes
	^ { TextColor red.  TextEmphasis bold }! !

!MagmaDoctor methodsFor: 'accessing' stamp: 'cmm 3/25/2007 14:25'!
writeHealthInformationTo: aWriteStream
	self
		printTitleOn: aWriteStream ;
		printPrimitiveAttributesOn: aWriteStream ;
		printCrucialBuffersOn: aWriteStream! !

MagmaWarning subclass: #MagmaToolsWarning
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma Tools-Exceptions'!

!MaObjectBuffer methodsFor: '*magma tools' stamp: 'cmm 3/19/2007 00:26'!
hasValidReferencesFor: aMaObjectFiler
	^ true! !

!MaObjectBuffer methodsFor: '*magma tools' stamp: 'cmm 3/19/2007 12:22'!
isValidBufferType
	^ self type between: 1 and: 5! !

!MaObjectBuffer methodsFor: '*magma tools' stamp: 'cmm 3/19/2007 00:18'!
isValidControlFields
	"The 4 high-order bits should be 0, since they're not used."
	| withZeroedLowOrderBits |
	withZeroedLowOrderBits _ self controlFields >> 4 << 4.
	^ withZeroedLowOrderBits = 0! !

!MaObjectBuffer methodsFor: '*magma tools' stamp: 'cmm 3/19/2007 12:21'!
isValidOid: anInteger for: aMaObjectFiler 
	^ anInteger
		between: MaOidCalculator firstUserObjectOid
		and: aMaObjectFiler getNextOid - 1! !

!MaObjectBuffer methodsFor: '*magma tools' stamp: 'cmm 3/19/2007 12:21'!
isValidOidFor: aMaObjectFiler 
	^ self 
		isValidOid: self oid
		for: aMaObjectFiler! !

PackageInfo subclass: #MagmaToolsPackageInfo
	instanceVariableNames: 'console'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma Tools-Info'!

!MagmaToolsPackageInfo methodsFor: 'as yet unclassified' stamp: 'cmm 5/8/2008 21:48'!
prerequisitePackages
	^ #('Magma server' )! !

!MaFixedObjectBuffer methodsFor: '*magma tools' stamp: 'cmm 3/19/2007 12:25'!
hasValidReferencesFor: aMaObjectFiler 
	self instVarsDoWithIndex: 
		[ : eachOid : x | 
		(self 
			isValidOid: eachOid
			for: aMaObjectFiler) ifFalse: [ ^ false ] ].
	^ true! !
